
from .data import Data

import contextlib, time



def set_trace(local):
    try:
        self = local.pop('self')
        __class__ = local.pop('__class__')
    except KeyError:
        pass
    local = Data(**local)
    locals().update(local.to_dict())
    import pdb
    import rlcompleter
    pdb.Pdb.complete = rlcompleter.Completer(locals()).complete
    pdb.set_trace()



def get_class_name(x):
    return str(type(x))[8:-2]


def get_method_name():
    import inspect
    f_name = inspect.getframeinfo(inspect.currentframe().f_back)[2]
    return f_name


def get_parent_class(x):
    return x.__base__


def get_class_parameters(x):
    """
        recursively
    """

    params = {}
    if get_parent_class(x) != None:
        parent_params = get_class_parameters(get_parent_class(x))
        params.update(parent_params)
    params.update(
        {key: value for key, value in x.__dict__.items() if not callable(value) and not hasattr(value, '__get__')}
    )
    return params




def spin(secs=5.0, func=None, args=(), kwargs={}):
    import time
    while True:
        if func != None:
            func(*args, **kwargs)
        time.sleep(secs)
    return


def prefix(x):
    res = ''
    if isinstance(x, str):
        res += x
    else:
        res += get_class_name(x)

    import inspect
    f_name = inspect.getframeinfo(inspect.currentframe().f_back)[2]
    res += '::' + f_name
    return '[' + res + '] '



def get_cpu_memory_usage(pid=None):
    if pid == None:
        import os
        pid = os.getpid()
    import psutil
    btypes = psutil.Process(os.getpid()).memory_info().rss
    return btypes


def get_gpu_memory_usage(pid=None):
    """
    Ref: https://github.com/gpuopenanalytics/pynvml/issues/21
    """
    if pid == None:
        import os
        pid = os.getpid()
    import pynvml
    pynvml.nvmlInit()
    btypes = 0
    for dev_id in range(pynvml.nvmlDeviceGetCount()):
        handle = pynvml.nvmlDeviceGetHandleByIndex(dev_id)
        for proc in pynvml.nvmlDeviceGetComputeRunningProcesses(handle):
            if proc.pid == pid:
                btypes += proc.usedGpuMemory
    return btypes



import memory_profiler
cpu_memory_profile = memory_profiler.profile



@contextlib.contextmanager
def time_measure(ident):
    tstart = time.time()
    yield
    elapsed = time.time() - tstart
    print("{0}: {1} s".format(ident, elapsed))




import sys
import trace
import linecache

class TraceLineExecution:
    def __enter__(self):
        self.tracer = trace.Trace(trace=0, count=0)
        self.sys_trace = sys.gettrace()
        sys.settrace(self.trace_lines)

    def __exit__(self, exc_type, exc_value, traceback):
        sys.settrace(self.sys_trace)

    def trace_lines(self, frame, event, arg):
        if event == 'line':
            code = frame.f_code
            line_no = frame.f_lineno
            filename = code.co_filename
            line = linecache.getline(filename, line_no).strip()
            print(f"Executing:  --->   File '{filename}', line {line_no}: {line}")
        return self.trace_lines




##############################################
############ file system #####################
##############################################


def mkdir(path):
    if not os.path.exists(path):
        os.makedirs(path)

def isdir(path):
    return os.path.exists(path) and not os.path.isfile(path)




##############################################
############ silent ##########################
##############################################


from contextlib import redirect_stdout
from io import StringIO 

class NullIO(StringIO):
    def write(self, txt):
        pass

def silent(fn):
    """Decorator to silence functions."""
    def silent_fn(*args, **kwargs):
        with redirect_stdout(NullIO()):
            return fn(*args, **kwargs)
    return silent_fn


import os, sys
class HiddenPrints:
    """
    with HiddenPrints():
        print("This wont print")
    """
    def __enter__(self):
        self._original_stdout = sys.stdout
        sys.stdout = open(os.devnull, 'w')

    def __exit__(self, exc_type, exc_val, exc_tb):
        sys.stdout.close()
        sys.stdout = self._original_stdout


